"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Gesstabs = void 0;
const node_xlsx_1 = __importDefault(require("node-xlsx"));
const parser_1 = require("./parser");
const csv = require('csv-parser');
const fs = require('fs');
class Gesstabs extends parser_1.Parser {
    constructor(config) {
        super(config);
    }
    fromCsv(file) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                fs.createReadStream(file)
                    .pipe(csv({ separator: ';', headers: false }))
                    .on('data', (data) => {
                    data = Object.values(data);
                    this.parseSections(data);
                })
                    .on('end', () => {
                    this.setDatasheets();
                    resolve(this.datasheets);
                });
            });
        });
    }
    fromXlsx(file) {
        return __awaiter(this, void 0, void 0, function* () {
            const workSheetsFromBuffer = node_xlsx_1.default.parse(fs.readFileSync(file));
            const data = workSheetsFromBuffer[0].data;
            data.forEach(row => {
                this.parseSections(row);
            });
            this.setDatasheets();
            return this.datasheets;
        });
    }
    parseSections(data) {
        const firstCell = String(data[0]).trim();
        const secondCell = data[1];
        const hasFirstCell = (data[0] && String(data[0]).length > 0);
        const hasSecondCell = (typeof (secondCell) !== 'undefined' && String(data[1]).length > 0);
        if (firstCell.indexOf(this.config.separator) !== -1) {
            this.count++;
            this.currentSection = 'info';
            this.results[this.count] = {
                info: [],
                header: [],
                body: [],
                meta: []
            };
        }
        if (firstCell.length === 0 && !hasSecondCell
            || this.config.skipRows.indexOf(firstCell) > -1) {
            return;
        }
        if (hasFirstCell && !hasSecondCell) {
            this.results[this.count].info.push({
                key: this.currentSection,
                value: firstCell
            });
        }
        if (!hasFirstCell && hasSecondCell
            && this.currentSection !== 'body') {
            this.currentSection = 'header';
            this.results[this.count].header.push(data.slice(1));
        }
        if (hasFirstCell && hasSecondCell) {
            this.currentSection = 'body';
            const isMeta = this.parseMeta(data, firstCell);
            if (!isMeta) {
                this.results[this.count].body.push(data);
            }
        }
    }
    parseMeta(data, firstCell) {
        for (const metaKey in this.config.metaMap) {
            const isMeta = this.config.metaMap[metaKey].find(metaLabel => firstCell.indexOf(metaLabel) === 0);
            if (isMeta) {
                this.results[this.count].meta.push({
                    key: metaKey,
                    label: firstCell,
                    data: data
                });
                return true;
            }
        }
        return false;
    }
}
exports.Gesstabs = Gesstabs;
//# sourceMappingURL=gesstabs.js.map