"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Parser = void 0;
class Parser {
    constructor(config) {
        this.results = [];
        this.datasheets = [];
        this.config = config;
        this.currentSection = '';
        this.count = -1;
    }
    setDatasheets() {
        for (const r in this.results) {
            const table = this.results[r];
            const tagsObj = {
                tags: [],
                push(cat, value) {
                    this.tags.push({
                        category: cat,
                        value: value
                    });
                }
            };
            this.config.renderTags(table.info, tagsObj);
            const tags = tagsObj.tags;
            const slices = this.parseColumnSlices(table.header);
            const subgroups = this.sliceColumns(table, table.header, slices);
            subgroups.forEach(subgroup => {
                const targetTags = [
                    ...tags,
                    this.getTag('subgroup', subgroup.label)
                ];
                this.datasheets.push({
                    tags: targetTags,
                    columns: subgroup.columns,
                    rows: subgroup.rows,
                    data: subgroup.data,
                    meta: subgroup.meta
                });
            });
        }
    }
    getTag(category, value) {
        const tag = {
            category: category,
            value: value
        };
        return tag;
    }
    parseColumnSlices(header) {
        const slices = [];
        const countHeaderLevels = header.length;
        const bottomLevel = countHeaderLevels - 1;
        const lastValues = {};
        const sliceKeys = {};
        header[bottomLevel].forEach((value, colId) => {
            const upperValues = [];
            for (let level = 0; level < bottomLevel; level++) {
                if (typeof (header[level][colId]) !== 'undefined'
                    && String(header[level][colId]).trim().length > 0) {
                    const upperValue = String(header[level][colId]);
                    upperValues.push(upperValue);
                    lastValues[level] = upperValue;
                }
                else if (lastValues[level]) {
                    upperValues.push(lastValues[level]);
                }
                else {
                    upperValues.push(value);
                }
            }
            const uniqueUpperValues = [...new Set(upperValues)];
            const sliceKey = uniqueUpperValues.join('|');
            if (!sliceKeys[sliceKey]) {
                sliceKeys[sliceKey] = [];
            }
            sliceKeys[sliceKey].push(colId);
        });
        for (const sliceKey in sliceKeys) {
            slices.push({
                label: sliceKey,
                start: sliceKeys[sliceKey][0],
                end: sliceKeys[sliceKey][0] + sliceKeys[sliceKey].length
            });
        }
        return slices;
    }
    sliceColumns(table, subgroupHeaders, slices) {
        const bottomLevel = subgroupHeaders.length - 1;
        const subgroupHeader = subgroupHeaders[bottomLevel];
        const body = table.body;
        const meta = table.meta;
        const subgroups = [];
        slices.forEach(slice => {
            const subgroup = {
                label: slice.label,
                rows: [],
                columns: [],
                data: [],
                meta: [],
            };
            body.forEach(row => {
                const rowArr = Object.values(row);
                const rowLabel = String(rowArr[0]);
                const sliced = rowArr.slice(slice.start + 1, slice.end + 1);
                subgroup.rows.push(rowLabel);
                subgroup.data.push(this.config.renderRow(sliced));
            });
            meta.forEach(meta => {
                subgroup.meta.push({
                    key: meta.key,
                    label: meta.label,
                    data: meta.data.slice(slice.start + 1, slice.end + 1)
                });
            });
            const colArr = Object.values(subgroupHeader);
            subgroup.columns = colArr.slice(slice.start, slice.end).map(col => String(col));
            subgroups.push(subgroup);
        });
        return subgroups;
    }
}
exports.Parser = Parser;
//# sourceMappingURL=parser.js.map