import { PrismaClient, Sheet, Tag } from "./client";
export declare type StoreOptions = {
    replaceExisting?: boolean;
    runBefore?: (prisma: PrismaClient) => Promise<void>;
};
export declare type ParserOptions = {
    separator: string;
    renderRow: (cells: ResultCell[]) => ResultCell[];
    renderTags: (info: RawResultInfo[], pushCb: Tagger) => void;
    metaMap: MetaMap;
    skipRows: string[];
};
export declare type ResultCell = number | string | null;
export declare type StoreSummary = {
    ids: number[];
    deleted: number[];
};
export declare type DataTag = {
    id?: number;
    category: string;
    value: string;
    categoryId?: number;
};
export declare type DataPoint = {
    tags: DataTag[];
    row: string;
    column: string;
    value: ResultCell;
    meta?: ResultCell;
};
export declare type Datasheet = {
    id?: number;
    tags: DataTag[];
    columns: string[];
    rows: string[];
    data: ResultCell[][];
    meta: RawResultMeta[];
};
export declare type TagWhereBySomeId = {
    tags: {
        some: {
            id: number;
        };
        AND?: TagWhereBySomeId;
    };
};
export declare type Sheets = (Sheet & {
    tags: Tag[];
})[];
export declare type DataPointFilterResult = {
    points: DataPoint[];
    label: string;
};
export declare type DataPointFilter = {
    (points: DataPoint[]): DataPointFilterResult;
};
export declare type DataGridFunction = {
    (tag: DataTag | string, key?: string): DataPointFilter;
};
export declare type DataGridCategories = {
    (keys: CellKeys): DataPointFilter[];
};
export declare type DataGrid = {
    rows: DataPointFilter[] | DataGridCategories;
    columns: DataPointFilter[] | DataGridCategories;
    cell: DataResultCellFilter;
    modify?: DataPointModifier[];
};
export declare type DataResultCellFilter = {
    (points: DataPoint[]): ResultCell;
};
export declare type DataPointModifier = {
    applyToLevel?: number[];
    cb: (points: DataPoint[], args?: any) => void;
};
export declare type CellKey = {
    [key: string]: boolean;
};
export declare type CellKeys = {
    [key: string]: CellKey;
};
export declare type ResultColumn = {
    key: string;
    value: ResultCell;
};
export declare type ResultRow = {
    key: string;
    cols: ResultColumn[];
};
export declare type Result = {
    body: ResultRow[];
};
export declare type Tagger = {
    tags: DataTag[];
    push: (cat: string, val: string) => void;
};
export declare type MetaMap = {
    [key: string]: string[];
};
export declare type RawRow = ResultCell[];
export declare type RawResultMeta = {
    key: string;
    label: string;
    data: RawRow | RawRow[];
};
export declare type RawResultInfo = {
    key: string;
    value: string;
};
export declare type RawResultData = {
    info: RawResultInfo[];
    header: RawRow[];
    body: RawRow[];
    meta: RawResultMeta[];
};
export declare type RawColumnSlice = {
    label: string;
    start: number;
    end: number;
    length: number;
};
export declare type RawTable = {
    label: string;
    rows: string[];
    columns: string[];
    data: ResultCell[][];
    meta: RawResultMeta[];
};
