"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
class TransformResult {
    constructor(result) {
        this.result = result;
    }
    forEachRow(cb) {
        this.result.body.forEach((row, r) => {
            cb(row, r);
        });
        return this;
    }
    forEachColumn(row, cb) {
        const targetRow = this.getRow(row);
        targetRow.cols.forEach((col, c) => {
            cb(col, c, targetRow);
        });
        return this;
    }
    forEachValue(cb) {
        this.forEachRow((row, r) => {
            this.forEachColumn(row, (col, c) => {
                const points = this.getDataPoints(col);
                cb(points, r, c, col);
            });
        });
        return this;
    }
    forEachDataPoint(cb) {
        this.forEachValue((points, r, c, col) => {
            points.forEach((point, id) => {
                cb(point, r, c, id);
            });
        });
        return this;
    }
    getRow(row) {
        return (typeof row === 'number')
            ? this.result.body[row]
            : row;
    }
    getDataPoints(col) {
        if (col.value && Array.isArray(col.value)) {
            return col.value;
        }
        return [];
    }
    getDataPoint(col, id) {
        const targetId = id || 0;
        const points = this.getDataPoints(col);
        if (points[targetId]) {
            return points[targetId];
        }
        return this.createDataPoint();
    }
    insertColumn(key, atIndex) {
        this.forEachRow((row, r) => {
            const leftCols = this.sliceColumns(row, 0, atIndex);
            const rightCols = this.sliceColumns(row, atIndex);
            const newCol = this.createColumn(row.key, key);
            row.cols = [
                ...leftCols,
                newCol,
                ...rightCols
            ];
        });
        return this;
    }
    createColumn(rowKey, colKey, value, points) {
        points = (!points) ? [
            this.createDataPoint(rowKey, colKey, value)
        ] : points;
        return {
            key: colKey,
            value: points,
            getPoint: __1.Query.getPointCb(points)
        };
    }
    createRow(rowKey, columns) {
        return {
            key: rowKey,
            cols: columns,
            getColumn: __1.Query.getColumnCb(columns)
        };
    }
    dropColumn(atIndex) {
        this.forEachRow((row, r) => {
            const leftCols = this.sliceColumns(row, 0, atIndex);
            const rightCols = this.sliceColumns(row, atIndex + 1);
            row.cols = [
                ...leftCols,
                ...rightCols
            ];
        });
        return this;
    }
    dropRow(atIndex) {
        const rowsBefore = this.result.body.slice(0, atIndex);
        const rowsAfter = this.result.body.slice(atIndex + 1);
        this.result.body = [
            ...rowsBefore,
            ...rowsAfter
        ];
        return this;
    }
    createDataPoint(rowKey, colKey, value) {
        const point = {
            tags: [],
            meta: [],
            row: rowKey || 'n/a',
            column: colKey || 'n/a',
            value: value || null,
            getMeta: () => undefined,
            getTag: () => undefined,
        };
        point.getMeta = __1.Query.getMetaCb(point);
        point.getTag = __1.Query.getTagCb(point);
        return point;
    }
    sliceRows(start, end) {
        this.result.body = this.result.body.slice(start, end);
    }
    sliceColumns(row, start, end) {
        const cols = row.cols.slice(start, end);
        return cols;
    }
    pushMeta(point, meta) {
        point.meta = (!point.meta) ? [] : point.meta;
        point.meta.push(meta);
    }
    setPointStyle(point, style) {
        if (!point)
            return;
        point.style = style;
    }
    dump(verbose) {
        this.forEachRow((row, r) => {
            console.log(row.key);
            this.forEachColumn(row, (col) => {
                if (verbose === true) {
                    console.log('   ' + col.key + ': ');
                    console.dir(this.getDataPoint(col), { depth: 10 });
                }
                else {
                    console.log('   ' + col.key + ': ' + this.getDataPoint(col).value);
                }
            });
        });
        console.log('--------------');
        return this;
    }
}
exports.default = TransformResult;
//# sourceMappingURL=transformResult.js.map