"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Parser = void 0;
const node_xlsx_1 = __importDefault(require("node-xlsx"));
const csv = require('csv-parser');
const fs = require('fs');
const v = (v) => {
    console.dir(v, { depth: 10 });
};
class Parser {
    constructor(config) {
        this.results = [];
        this.datasheets = [];
        this.config = config;
        this.currentSection = '';
        this.count = -1;
    }
    storeCsvFile(file, store) {
        fs.createReadStream(file)
            .pipe(csv({ separator: ';', headers: false }))
            .on('data', (data) => {
            data = Object.values(data);
            this.parseSections(this.config, data);
        })
            .on('end', () => __awaiter(this, void 0, void 0, function* () {
            this.setDatasheets();
            yield store.run(this.datasheets);
            console.log(store.summary);
        }));
    }
    storeXlsxFile(file, store) {
        return __awaiter(this, void 0, void 0, function* () {
            const workSheetsFromBuffer = node_xlsx_1.default.parse(fs.readFileSync(file));
            const data = workSheetsFromBuffer[0].data;
            data.forEach(row => {
                this.parseSections(this.config, row);
            });
            this.setDatasheets();
            yield store.run(this.datasheets);
            return store.summary;
        });
    }
    parseSections(config, data) {
        const firstCell = String(data[0]).trim();
        const secondCell = data[1];
        const hasFirstCell = (data[0] && String(data[0]).length > 0);
        const hasSecondCell = (typeof (secondCell) !== 'undefined' && String(data[1]).length > 0);
        if (firstCell.indexOf(config.separator) !== -1) {
            this.count++;
            this.currentSection = 'info';
            this.results[this.count] = {
                info: [],
                header: [],
                body: [],
                meta: []
            };
        }
        if (firstCell.length === 0 && !hasSecondCell
            || config.skipRows.indexOf(firstCell) > -1) {
            return;
        }
        if (hasFirstCell && !hasSecondCell) {
            this.results[this.count].info.push({
                key: this.currentSection,
                value: firstCell
            });
        }
        if (!hasFirstCell && hasSecondCell
            && this.currentSection !== 'body') {
            this.currentSection = 'header';
            this.results[this.count].header.push(data.slice(1));
        }
        if (hasFirstCell && hasSecondCell) {
            this.currentSection = 'body';
            const isMeta = this.parseMeta(data, firstCell);
            if (!isMeta) {
                this.results[this.count].body.push(data);
            }
        }
    }
    parseMeta(data, firstCell) {
        for (const metaKey in this.config.metaMap) {
            const isMeta = this.config.metaMap[metaKey].find(metaLabel => firstCell.indexOf(metaLabel) === 0);
            if (isMeta) {
                this.results[this.count].meta.push({
                    key: metaKey,
                    label: firstCell,
                    data: data
                });
                return true;
            }
        }
        return false;
    }
    setDatasheets() {
        for (const r in this.results) {
            const table = this.results[r];
            const tagsObj = {
                tags: [],
                push(cat, value) {
                    this.tags.push({
                        category: cat,
                        value: value
                    });
                }
            };
            this.config.renderTags(table.info, tagsObj);
            const tags = tagsObj.tags;
            const slices = this.parseColumnSlices(table.header);
            const subgroups = this.sliceColumns(table, table.header, slices);
            subgroups.forEach(subgroup => {
                const targetTags = [
                    ...tags,
                    this.getTag('subgroup', subgroup.label)
                ];
                this.datasheets.push({
                    tags: targetTags,
                    columns: subgroup.columns,
                    rows: subgroup.rows,
                    data: subgroup.data,
                    meta: subgroup.meta
                });
            });
        }
    }
    getTag(category, value) {
        const tag = {
            category: category,
            value: value
        };
        return tag;
    }
    parseColumnSlices(header) {
        const slices = [];
        const countHeaderLevels = header.length;
        const bottomLevel = countHeaderLevels - 1;
        const lastValues = {};
        const sliceKeys = {};
        header[bottomLevel].forEach((value, colId) => {
            const upperValues = [];
            for (let level = 0; level < bottomLevel; level++) {
                if (typeof (header[level][colId]) !== 'undefined'
                    && String(header[level][colId]).trim().length > 0) {
                    const upperValue = String(header[level][colId]);
                    upperValues.push(upperValue);
                    lastValues[level] = upperValue;
                }
                else if (lastValues[level]) {
                    upperValues.push(lastValues[level]);
                }
                else {
                    upperValues.push(value);
                }
            }
            const uniqueUpperValues = [...new Set(upperValues)];
            const sliceKey = uniqueUpperValues.join('|');
            if (!sliceKeys[sliceKey]) {
                sliceKeys[sliceKey] = [];
            }
            sliceKeys[sliceKey].push(colId);
        });
        for (const sliceKey in sliceKeys) {
            slices.push({
                label: sliceKey,
                start: sliceKeys[sliceKey][0],
                end: sliceKeys[sliceKey][0] + sliceKeys[sliceKey].length
            });
        }
        return slices;
    }
    sliceColumns(table, subgroupHeaders, slices) {
        const bottomLevel = subgroupHeaders.length - 1;
        const subgroupHeader = subgroupHeaders[bottomLevel];
        const body = table.body;
        const meta = table.meta;
        const subgroups = [];
        slices.forEach(slice => {
            const subgroup = {
                label: slice.label,
                rows: [],
                columns: [],
                data: [],
                meta: [],
            };
            body.forEach(row => {
                const rowArr = Object.values(row);
                const rowLabel = String(rowArr[0]);
                const sliced = rowArr.slice(slice.start + 1, slice.end + 1);
                subgroup.rows.push(rowLabel);
                subgroup.data.push(this.config.renderRow(sliced));
            });
            meta.forEach(meta => {
                subgroup.meta.push({
                    key: meta.key,
                    label: meta.label,
                    data: meta.data.slice(slice.start + 1, slice.end + 1)
                });
            });
            const colArr = Object.values(subgroupHeader);
            subgroup.columns = colArr.slice(slice.start, slice.end).map(col => String(col));
            subgroups.push(subgroup);
        });
        return subgroups;
    }
}
exports.Parser = Parser;
//# sourceMappingURL=parser.js.map