"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Gesstabs = void 0;
const node_xlsx_1 = __importDefault(require("node-xlsx"));
const parser_1 = require("./parser");
const csv = require('csv-parser');
const fs = require('fs');
const path = require('path');
class Gesstabs extends parser_1.Parser {
    constructor(config) {
        super(config);
        this.parseSigniHeader = (cells, meta) => {
            var _a;
            if (!this.config.significance)
                return cells;
            const headerSeparator = this.config.significance.headerSeparator || '$';
            const signiMeta = {
                key: this.config.significance.lettersKey,
                label: this.config.significance.lettersKey,
                info: [],
            };
            cells.forEach((cell, c) => {
                var _a;
                const split = cell.split(headerSeparator);
                if (split[1]) {
                    (_a = signiMeta.info) === null || _a === void 0 ? void 0 : _a.push({
                        key: split[0],
                        value: split[1]
                    });
                    cells[c] = split[0];
                }
            });
            if ((_a = signiMeta.info) === null || _a === void 0 ? void 0 : _a.length) {
                meta.push(signiMeta);
            }
        };
        this.parseSigniValues = (cell, colId, rowLabel, meta, signiHeaderLetters) => {
            if (!this.config.significance)
                return cell;
            const signiCellSeparator = this.config.significance.cellSeparator;
            const split = cell.split(signiCellSeparator);
            if (split[1]) {
                const signiMap = this.getSigniMap(split[1], signiHeaderLetters, colId);
                if (signiMap.length) {
                    const valueKey = this.config.significance.valueKey;
                    this.pushMeta(rowLabel, valueKey, signiMap, meta);
                }
                cell = split[0];
            }
            return cell;
        };
        this.getSigniMap = (signiCellInfo, signiHeaderLetters, colId) => {
            var _a;
            const signiHeaderFilter = ((_a = this.config.significance) === null || _a === void 0 ? void 0 : _a.headerFilter) || [];
            const signiLetters = signiCellInfo.split('');
            const signiMap = [];
            signiLetters.forEach((signiLetter) => {
                const thisHeader = signiHeaderLetters.find(singiHeaderLetter => singiHeaderLetter.value === signiLetter);
                const thisKey = signiHeaderLetters[colId].key;
                const vsKey = thisHeader === null || thisHeader === void 0 ? void 0 : thisHeader.key;
                if (vsKey && (signiHeaderFilter.includes(vsKey) || signiHeaderFilter.includes(thisKey))) {
                    this.pushSigniMap(signiMap, thisKey, 'higherThan', vsKey);
                    this.pushSigniMap(signiMap, vsKey, 'lowerThan', thisKey);
                }
            });
            return signiMap;
        };
    }
    fromCsv(file) {
        return __awaiter(this, void 0, void 0, function* () {
            this.file = path.basename(file);
            return new Promise((resolve, reject) => {
                fs.createReadStream(file)
                    .pipe(csv({ separator: ';', headers: false }))
                    .on('data', (data) => {
                    data = Object.values(data);
                    this.parseSections(data);
                })
                    .on('end', () => {
                    this.setDatasheets();
                    resolve(this.datasheets);
                });
            });
        });
    }
    fromXlsx(file) {
        return __awaiter(this, void 0, void 0, function* () {
            this.file = path.basename(file);
            const workSheetsFromBuffer = node_xlsx_1.default.parse(fs.readFileSync(file));
            const data = workSheetsFromBuffer[0].data;
            console.log('File rows count: ' + String(data.length));
            this.autoDetectConfig(data);
            console.log('Table separator: ' + this.tableSeparator);
            data.forEach(row => {
                this.parseSections(row);
            });
            this.setDatasheets();
            return this.datasheets;
        });
    }
    autoDetectConfig(data) {
        if (!this.config.separator) {
            const tableSeparatorRow = data.find((row) => row[0]
                && typeof row[0] === 'string'
                && row[0].trim().length > 0);
            this.tableSeparator = tableSeparatorRow[0];
        }
        else {
            this.tableSeparator = this.config.separator;
        }
    }
    parseSections(data) {
        var _a;
        const firstCell = String(data[0]).trim();
        const secondCell = data[1];
        const hasFirstCell = (data[0] && String(data[0]).length > 0);
        const hasSecondCell = (typeof (secondCell) !== 'undefined' && String(data[1]).length > 0);
        if (firstCell.indexOf(this.tableSeparator) !== -1) {
            this.count++;
            this.currentSection = 'info';
            this.results[this.count] = {
                info: [{
                        key: 'file',
                        value: this.file
                    }],
                header: [],
                body: [],
                meta: [],
                nested: [],
            };
            this.nested = [];
        }
        if (firstCell.length === 0 && !hasSecondCell
            || this.config.skipRows.indexOf(firstCell) > -1) {
            return;
        }
        if (hasFirstCell && !hasSecondCell) {
            this.results[this.count].info.push({
                key: this.currentSection,
                value: firstCell
            });
        }
        if (!hasFirstCell && hasSecondCell
            && this.currentSection !== 'body') {
            this.currentSection = 'header';
            this.results[this.count].header.push(data.slice(1));
        }
        if (hasFirstCell && hasSecondCell) {
            this.currentSection = 'body';
            const isMeta = this.parseMeta(data, firstCell);
            if (!isMeta) {
                this.results[this.count].body.push(data);
                this.parseNested(data, firstCell);
                (_a = this.results[this.count].nested) === null || _a === void 0 ? void 0 : _a.push({
                    parents: this.nested,
                    label: firstCell
                });
            }
        }
    }
    parseMeta(data, firstCell) {
        for (const metaKey in this.config.metaMap) {
            const isMeta = this.config.metaMap[metaKey].find(metaLabel => firstCell.indexOf(metaLabel) === 0);
            if (isMeta) {
                this.results[this.count].meta.push({
                    key: metaKey,
                    label: firstCell,
                    data: data
                });
                return true;
            }
        }
        return false;
    }
    parseNested(data, firstCell) {
        if (!this.config.overcodes)
            return false;
        for (const overcodeLevel in this.config.overcodes) {
            const overcode = this.config.overcodes[overcodeLevel];
            let isOvercode = false;
            if (overcode.prefix) {
                isOvercode = (firstCell.indexOf(overcode.prefix) === 0);
            }
            else if (overcode.match) {
                isOvercode = (firstCell.includes(overcode.match));
            }
            else if (overcode.callback) {
                isOvercode = overcode.callback(firstCell);
            }
            if (isOvercode) {
                this.nested = this.nested.filter(nested => nested.level < Number(overcodeLevel));
                const parent = {
                    level: Number(overcodeLevel),
                    label: firstCell,
                    key: overcode.key,
                };
                this.nested.push(parent);
                return true;
            }
        }
        return false;
    }
    pushSigniMap(signiMap, value, info, key) {
        signiMap.push({
            value: value,
            info: info,
            key: key,
        });
    }
}
exports.Gesstabs = Gesstabs;
//# sourceMappingURL=gesstabs.js.map