"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helper_1 = require("./helper");
class Points {
    constructor(points) {
        this.points = points;
    }
    targetPoints(points) {
        if (points !== undefined) {
            return points;
        }
        return this.points;
    }
    filter(args) {
        const { key, values, replace, origin } = args;
        if (Array.isArray(values) && values.length) {
            let points;
            if (key === 'row' || key === 'column') {
                points = this.points.filter(point => {
                    const targetPoint = this.getTargetPoint(point, origin);
                    return values.includes(targetPoint[key]);
                });
            }
            else {
                points = this.points.filter(point => {
                    const targetPoint = this.getTargetPoint(point, origin);
                    return targetPoint.tags.find(tag => tag.categoryId === Number(key) && values.includes(tag.value));
                });
            }
            if (replace === true) {
                this.replace(points);
            }
            return points;
        }
        return this.points;
    }
    getTargetPoint(point, origin) {
        if (origin === true) {
            if (point.origin && point.origin[0])
                return point.origin[0];
        }
        return point;
    }
    filterNested(args) {
        const { values, hideParents, hideChildren } = args;
        if (Array.isArray(values) && values.length) {
            this.filterMetaByNested((metaValues, point) => metaValues.some(el => values.includes(el)));
        }
        if (hideParents !== false) {
            this.filterMetaByNested((metaValues, point) => !metaValues.includes(point.row));
        }
        else if (hideChildren !== false) {
            this.filterMetaByNested((metaValues, point) => metaValues.includes(point.row));
        }
    }
    filterMetaByNested(cb) {
        let points = this.points.filter(point => {
            var _a;
            return (_a = point.meta) === null || _a === void 0 ? void 0 : _a.find(meta => meta.key === 'nested'
                && Array.isArray(meta.value)
                && cb(meta.value.map((parent) => parent.label), point));
        });
        this.replace(points);
    }
    exclude(args) {
        const { key, values, excludeAll } = args;
        let points = [];
        if (key === 'row' || key === 'column') {
            if (values.length) {
                points = this.points.filter(point => !values.includes(point[key]));
            }
        }
        else {
            if (excludeAll !== true) {
                points = this.points.filter(point => point.tags.find(tag => tag.categoryId === Number(key) && !values.includes(tag.value)));
            }
            else {
                points = this.points.filter(point => {
                    return !point.tags.find(tag => tag.categoryId === Number(key));
                });
            }
        }
        this.replace(points);
    }
    addStringToLabel(args, points) {
        const { value, target } = args;
        this.targetPoints(points).forEach(point => {
            this.pushPointOrigin(point);
            point[target] += value;
        });
    }
    addTagToLabel(args, points) {
        const { categoryId, target } = args;
        const glue = (!args.glue) ? ' ' : args.glue;
        this.targetPoints(points).forEach(point => {
            this.pushPointOrigin(point);
            const tag = point.tags.find(tag => tag.categoryId === categoryId);
            const value = (tag === null || tag === void 0 ? void 0 : tag.value) ? tag === null || tag === void 0 ? void 0 : tag.value : 'n/a';
            point[target] += glue + value;
        });
    }
    addToOthers(args, points) {
        const { match, mode } = args;
        const insert = (match === 'row') ? 'column' : 'row';
        const addTo = [];
        points.forEach(addPoint => {
            this.points.forEach(point => {
                if (addPoint[match] === point[match] && addPoint[insert] !== point[insert]) {
                    const addToPoints = Object.assign({}, addPoint);
                    addToPoints[insert] = point[insert];
                    addToPoints.mode = mode;
                    addTo.push(addToPoints);
                }
            });
        });
        this.push(addTo);
        return addTo;
    }
    addToNew(args, points) {
        const { key, values, alias, mode } = args;
        points.forEach((point, index) => {
            this.pushPointOrigin(point);
            const newPoint = Object.assign({}, point);
            newPoint[key] = alias;
            newPoint.mode = mode;
            this.points.push(newPoint);
        });
    }
    addMeta(args, points) {
        const { key } = args;
        const replacePoints = [];
        this.targetPoints(points).forEach(point => {
            var _a;
            this.pushPointOrigin(point);
            const targetMeta = (_a = point.meta) === null || _a === void 0 ? void 0 : _a.find(meta => meta.key === key);
            if (targetMeta && targetMeta.value) {
                if (args.append) {
                    const newPoint = Object.assign({}, point);
                    newPoint.value = targetMeta.value;
                    newPoint.row = newPoint.row + ' ' + targetMeta.key;
                    this.push([newPoint]);
                }
                else {
                    if (args.glue) {
                        point.value += String(args.glue) + targetMeta.value;
                    }
                    else if (args.replace) {
                        point.value = targetMeta.value;
                        point.row = targetMeta.key;
                        this.pushUnique(replacePoints, point);
                    }
                    else {
                        point.value = targetMeta.value;
                    }
                }
            }
        });
        if (replacePoints.length) {
            this.replace(replacePoints);
        }
    }
    addPointInfo(args) {
        this.points.forEach(point => {
        });
    }
    pushUnique(points, point) {
        const existing = points.find(existingPoint => existingPoint.row === point.row
            && existingPoint.column === point.column);
        if (!existing) {
            points.push(point);
        }
    }
    map(args, points) {
        const { source, target } = args;
        this.targetPoints(points).forEach(point => {
            this.pushPointOrigin(point);
            if (source === 'row' || source === 'column') {
                point[target] = point[source];
            }
            else {
                const tag = point.tags.find(tag => tag.categoryId === source);
                point[target] = (tag === null || tag === void 0 ? void 0 : tag.value) ? tag === null || tag === void 0 ? void 0 : tag.value : 'n/a';
            }
        });
    }
    transpose(args, points) {
        this.targetPoints(points).forEach(point => {
            this.pushPointOrigin(point);
            const row = point.row;
            point.row = point.column;
            point.column = row;
        });
    }
    calcDifference(args, points) {
        const { match, item } = args;
        const oppoMatch = (match === 'row') ? 'column' : 'row';
        this.targetPoints(points).forEach(point => {
            const vsPoint = this.points.find(vsPoint => point !== vsPoint
                && point[oppoMatch] === vsPoint[oppoMatch]
                && vsPoint[match] === item);
            if (vsPoint) {
                this.pushPointOrigin(point);
                if (args.revert === true) {
                    point.value = Number(vsPoint.value) - Number(point.value);
                }
                else {
                    point.value = Number(point.value) - Number(vsPoint.value);
                }
            }
        });
    }
    calcSum(args, points) {
        const { match, items, alias } = args;
        const oppoMatch = (match === 'row') ? 'column' : 'row';
        const sumStacks = [];
        items === null || items === void 0 ? void 0 : items.forEach(item => {
            const sumPoints = this.points.filter(vsPoint => vsPoint[match] === item);
            sumPoints.forEach(sumPoint => {
                const existing = sumStacks.find(existingSumPoint => existingSumPoint.alias === alias
                    && existingSumPoint.key === sumPoint[oppoMatch]);
                this.pushToStack(existing, sumPoint, sumStacks, alias, sumPoint[oppoMatch]);
            });
        });
        sumStacks.forEach(sumStack => {
            this.pushToPoints(sumStack, match, alias, this.targetPoints(points));
        });
    }
    pushToStack(existing, sumPoint, sumStacks, alias, key) {
        if (!existing) {
            sumStacks.push({
                alias: alias,
                key: key,
                points: [sumPoint]
            });
        }
        else {
            existing.points.push(sumPoint);
        }
    }
    pushToPoints(sumStack, match, alias, points) {
        const newItem = Object.assign({}, sumStack.points[0]);
        newItem[match] = alias;
        newItem.value = 0;
        sumStack.points.forEach((sumPoint) => {
            newItem.value = Number(newItem.value) + Number(sumPoint.value);
        });
        points.push(newItem);
    }
    rename(args, points) {
        const { renameStack } = args;
        this.targetPoints(points).forEach(point => {
            this.pushPointOrigin(point);
            renameStack.forEach(rename => {
                if (rename.isPattern && rename.isPattern === true) {
                    rename.cb = (label) => {
                        const regExp = new RegExp(rename.replace);
                        return label.replace(regExp, rename.by);
                    };
                }
                let targets = this.getRenameTargets(rename);
                this.applyRenameLabel(targets, rename, point);
            });
        });
    }
    getRenameTargets(rename) {
        let targets = [];
        if (rename.target) {
            targets.push(rename.target);
        }
        if (rename.targets) {
            targets = rename.targets;
        }
        return targets;
    }
    applyRenameLabel(targets, rename, point) {
        targets.forEach(target => {
            if (rename.cb) {
                point[target] = rename.cb(point[target]);
            }
            else if (rename.replace && rename.by) {
                if (point[target] && point[target] === rename.replace) {
                    point[target] = rename.by;
                }
            }
            if (rename.ucFirst) {
                point[target] = point[target][0].toUpperCase() + point[target].slice(1);
            }
        });
    }
    replace(replaceBy) {
        this.points.length = 0;
        this.push(replaceBy);
    }
    push(addPoints) {
        this.points.push(...addPoints);
    }
    pushPointOrigin(point) {
        point.origin = (!point.origin)
            ? [Object.assign({}, point)] : [...point.origin, Object.assign({}, point)];
    }
    dump(points) {
        helper_1.vd(this.targetPoints(points));
    }
}
exports.default = Points;
//# sourceMappingURL=points.js.map