import { Tag } from './client';
import { PrismaClient } from './client';
import { Datasheet, DataTag, Sheets, DataGrid, DataPoint, Result, CellKeys, DataPointFilter, DataPointModifier, DataPointSortation, DataGridTransformation, Selector, RawResultMeta, DataPointMeta, SelectionValidator, QueryOptions, ResultColumn, CategoryCount } from './types';
export default class Query {
    prisma: PrismaClient | any;
    clause: any;
    sheets: Datasheet[];
    allSheets: Datasheet[];
    inputKeys: CellKeys;
    keys: CellKeys;
    visibleKeys: {
        row: string[];
        column: string[];
    };
    points: DataPoint[];
    grid: DataGrid;
    selectionValidator: SelectionValidator;
    result: Result;
    tags: any[];
    nonGreedySelector: boolean;
    constructor(prisma: PrismaClient | any);
    setGrid(grid: DataGrid): this;
    setOptions(options?: QueryOptions): this;
    setSelectionValidator(validator: SelectionValidator): void;
    get(tags: DataTag[] | DataTag[][]): Promise<Query>;
    getByIds(allTagIds: Selector): Promise<Query>;
    processSheets(sheets: Sheets, level: number): void;
    getTagInfo(tagIds: number[]): Promise<Tag[]>;
    getSheets(tagStrings: DataTag[]): Promise<Sheets>;
    getSheetsByTags(tags: Tag[]): Promise<Sheets>;
    getSheetsById(ids: number[]): Promise<Sheets>;
    findSheets(tags: Tag[]): Promise<Sheets>;
    parseSheets(sheets: Sheets): void;
    filterSheets(sheets: Sheets): Sheets;
    getSheetCategoryCount(sheets: Sheets): CategoryCount[];
    setDataPoints(dataPoints: DataPoint[]): DataPoint[];
    getDataPointMeta(sheet: Datasheet, r: number, c: number): DataPointMeta[];
    pushPointInfo(metaContent: RawResultMeta, pointMeta: DataPointMeta[], sheet: Datasheet, r: number, c: number): void;
    pushPointNestedMeta(metaContent: any, pointMeta: any, sheet: any, r: number, c: number): void;
    metaHasRows(metaData: any): boolean;
    pushPointMeta(pointMeta: DataPointMeta[], key: string, value: any): void;
    modifyDataPoints(dataPoints: DataPoint[], level: number): void;
    getDatapointModifiersByLevel(level: number): DataPointModifier[];
    pushDataPoints(dataPoints: DataPoint[]): void;
    setDataPointKeys(dataPoints: DataPoint[], mode: 'keys' | 'inputKeys'): void;
    addNestedKeys(point: DataPoint, mode: 'keys' | 'inputKeys'): void;
    addKey(category: string, value: string, mode: 'keys' | 'inputKeys'): void;
    merge(): Promise<this>;
    clone(): this;
    checkForCallback(cb: any, keys: CellKeys, points: DataPoint[]): DataPointFilter[];
    getTagIds(tags: DataTag[]): Promise<number[]>;
    getTags(tags: DataTag[]): Promise<DataTag[]>;
    setCategoryId(tag: DataTag): Promise<void>;
    setTagId(tag: DataTag): Promise<void>;
    setResult(result: any): void;
    static getPointCb(points: DataPoint[]): (index?: number) => DataPoint;
    static getColumnCb(cols: ResultColumn[]): (colId?: number) => ResultColumn;
    static getMetaCb(point: DataPoint): (key: string) => DataPointMeta | undefined;
    static getTagCb(point: DataPoint): (categoryId: number) => DataTag | undefined;
    filterColumns(columns: number | number[]): Query;
    sort(cb: any): Query;
    sortResult(sortation: DataPointSortation[]): void;
    transformResult(transformations: DataGridTransformation[]): Promise<void>;
}
