"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Result {
    constructor(query) {
        this.getSeries = () => {
            return this.result.body[0].cols.map(col => {
                return {
                    label: col.key,
                    style: col.style
                };
            });
        };
        this.result = query.result;
        this.inputKeys = query.inputKeys;
        this.keys = query.keys;
        this.visibleKeys = query.visibleKeys;
        this.allSheets = query.allSheets;
        this.tags = [];
        this.metaParams = {};
    }
    setMetaParams(params) {
        this.metaParams = params;
    }
    toSeriesCategories() {
        if (this.result.body[0]) {
            const series = this.getSeries();
            const categories = [];
            this.result.body.forEach(row => {
                categories.push({
                    label: row.key,
                    values: row.cols.map(column => this.toNumber(column)),
                    styles: this.applyStyleCallback(row)
                });
            });
            return {
                series: series,
                categories: categories
            };
        }
        return {
            series: [],
            categories: []
        };
    }
    toVerticalLines() {
        const series = this.getSeries();
        const categories = [];
        this.result.body.forEach((row, r) => {
            categories.push({
                label: row.key,
                y: r,
                values: row.cols.map(cell => this.toNumber(cell)),
            });
        });
        return {
            series: series,
            categories: categories
        };
    }
    toCombo() {
        const series = this.getSeries();
        const categories = [];
        this.result.body.forEach((row, r) => {
            categories.push({
                label: row.key,
                y: r + 0.5,
                values: row.cols.map(cell => this.toNumber(cell)),
                styles: this.parseValueStyle(row.cols)
            });
        });
        return {
            series: series,
            categories: categories
        };
    }
    toScatter() {
        const series = this.getSeries();
        const categories = [];
        this.result.body.forEach((row, r) => {
            categories.push({
                label: row.key,
                values: row.cols.map((col) => {
                    return {
                        x: Number(col.value[0].value),
                        y: Number(col.value[1].value)
                    };
                }),
                styles: this.applyStyleCallback(row)
            });
        });
        return {
            series: series,
            categories: categories
        };
    }
    toBubbles() {
        const series = this.getSeries();
        const categories = [];
        this.result.body.forEach((row, r) => {
            categories.push({
                label: row.key,
                values: row.cols.map((col) => {
                    return {
                        size: Number(col.value[0].value),
                        x: Number(col.value[1].value),
                        y: Number(col.value[2].value),
                    };
                }),
                styles: this.applyStyleCallback(row)
            });
        });
        return {
            series: series,
            categories: categories
        };
    }
    toLabels() {
        return {
            body: this.result.body.map(row => {
                return {
                    values: [row.key],
                    styles: this.applyStyleCallback(row)
                };
            })
        };
    }
    toRowLabels() {
        return this.toLabels();
    }
    toColumnLabels() {
        let series = [];
        let styles = [];
        if (this.result.body && this.result.body.length) {
            series = this.result.body[0].cols.map(col => col.key);
            styles = this.applyStyleCallback(this.result.body[0]);
        }
        return {
            body: [
                {
                    values: series,
                    styles: styles
                }
            ]
        };
    }
    toTableBody() {
        return {
            body: this.result.body.map(row => {
                return {
                    values: row.cols.map(cell => this.toNumber(cell))
                };
            }),
        };
    }
    toTable(params) {
        const body = [];
        if (!this.result.body[0]) {
            return {
                body: [
                    {
                        values: []
                    }
                ]
            };
        }
        if (params === null || params === void 0 ? void 0 : params.showColumnLabels) {
            const header = [];
            if (params === null || params === void 0 ? void 0 : params.showRowLabels) {
                header.push('');
            }
            header.push(...this.result.body[0].cols.map(col => col.key));
            body.push({
                values: header
            });
        }
        this.result.body.forEach((row, r) => {
            const tableRow = [];
            if (params === null || params === void 0 ? void 0 : params.showRowLabels) {
                tableRow.push(String(row.key));
            }
            row.cols.forEach((cell, c) => {
                tableRow.push(this.renderCellValue(cell));
            });
            body.push({
                values: tableRow,
                styles: this.applyStyleCallback(row)
            });
        });
        return {
            body: body
        };
    }
    toNumber(column) {
        const value = this.renderCellValue(column);
        if (value === '')
            return value;
        return Number(value);
    }
    renderCellValue(column) {
        if (!column)
            return '';
        if (Array.isArray(column.value)) {
            if (!column.value[0] || column.value[0].value === null) {
                return '';
            }
            if (column.value.length > 0) {
                return column.value[0].value;
            }
        }
        if (typeof column.value === 'number'
            || typeof column.value === 'string') {
            return column.value;
        }
        return '';
    }
    formatPointKeys(keys) {
        const rowOrColumn = ['row', 'column', 'nested'];
        const queryResultKeys = {
            row: (keys.row) ? Object.keys(keys.row) : [],
            column: (keys.column) ? Object.keys(keys.column) : [],
            nested: (keys.nested) ? Object.keys(keys.nested) : [],
            category: []
        };
        for (const key in keys) {
            if (!rowOrColumn.includes(key)) {
                queryResultKeys.category.push({
                    categoryId: Number(key),
                    keys: Object.keys(keys[key])
                });
            }
        }
        return queryResultKeys;
    }
    applyStyleCallback(row) {
        var _a;
        if ((_a = this.metaParams) === null || _a === void 0 ? void 0 : _a.cb) {
            return this.metaParams.cb(row, this.metaParams, this);
        }
        const styles = [];
        row.cols.forEach((column) => {
            if (column.value && Array.isArray(column.value)) {
                column.value.forEach(point => {
                    if (point.style) {
                        styles.push(point.style);
                    }
                    else {
                        styles.push(null);
                    }
                });
            }
        });
        return styles;
    }
    parseValueStyle(cols) {
        return cols.map(cell => {
            if (Array.isArray(cell.value)
                && cell.value[0]
                && cell.value[0].style) {
                return cell.value[0].style;
            }
            return null;
        });
    }
}
exports.default = Result;
//# sourceMappingURL=result.js.map