import { PrismaClient, Sheet, Tag } from "./client";
import { Gesstabs } from "./parser/gesstabs";
import { Generic } from ".";
import { Parser } from './parser/parser';
import ResultClass from './result';
import { TableRowStyle } from 'pptx-automizer';
import { ChartValueStyle } from 'pptx-automizer';
import ResultInfo from './helper/resultInfo';
import TransformResult from './helper/transformResult';
export declare type StoreOptions = {
    replaceExisting?: boolean;
    runBefore?: (prisma: PrismaClient) => Promise<void>;
};
export declare type ParserType = Parser | Generic | Gesstabs;
export declare type ParserOptions = {
    separator?: string;
    firstCell?: string;
    renderTags: (info: RawResultInfo[], pushCb: Tagger) => void;
    renderHeader?: (cells: string[], meta: RawResultMeta[], parser: ParserType) => string[];
    renderRow: (cells: ResultCell[], label: string, meta: RawResultMeta[], parser: ParserType) => ResultCell[];
    renderLabel?: (label: string) => string;
    renderRawResultData?: (data: RawResultData, parser: ParserType) => void;
    metaMap: MetaMap;
    significance?: ParserOptionsSignificance;
    overcodes?: Overcodes[];
    skipRows: string[];
    worksheetId?: number;
};
export declare type ParserOptionsSignificance = {
    cellSeparator: string;
    headerSeparator: string;
    headerFilter: string[];
    lettersKey: string;
    valueKey: string;
};
export declare type ResultCell = number | string | null;
export declare type StoreSummary = {
    ids: number[];
    deleted: number[];
};
export declare type QueryOptions = {
    selectionValidator?: SelectionValidator;
    nonGreedySelector?: boolean;
};
export declare type SelectionValidator = {
    (tags: Tag[]): boolean;
};
export declare type CategoryCount = {
    sheetId: number;
    categoryIds: number[];
};
export declare type Selector = number[][];
export declare type DataTag = {
    id?: number;
    category: string;
    value: string;
    categoryId?: number;
};
export declare type DataPoint = {
    tags: DataTag[];
    row: string;
    column: string;
    value: ResultCell;
    meta?: DataPointMeta[];
    origin?: DataPoint[];
    mode?: string;
    style?: TableRowStyle | ChartValueStyle;
    getMeta: (key: string) => DataPointMeta | undefined;
    getTag: (categoryId: number) => DataTag | undefined;
};
export declare type DataPointMeta = {
    key: string;
    value: ResultCell | NestedParentValue[];
};
export declare type Datasheet = {
    id?: number;
    tags: DataTag[];
    columns: string[];
    rows: string[];
    data: ResultCell[][];
    meta: RawResultMeta[];
};
export declare type Sheets = (Sheet & {
    tags: Tag[];
})[];
export declare type QueryResult = {
    result: string;
    sheets: string;
    tags: string;
    keys: string;
    inputKeys: string;
    visibleKeys: string;
};
export declare type QueryResultKeys = {
    row: string[];
    column: string[];
    category: QueryResultCategoryKeys[];
};
export declare type QueryResultCategoryKeys = {
    categoryId: number;
    keys: string[];
};
export declare type DataPointFilterResult = {
    points: DataPoint[];
    label: string;
};
export declare type DataPointFilter = {
    (points: DataPoint[]): DataPointFilterResult;
};
export declare type DataGridFunction = {
    (tag: DataTag | string, key?: string | string[]): DataPointFilter;
};
export declare type DataGridCategories = {
    (keys: CellKeys): DataPointFilter[];
};
export declare type DataGrid = {
    row: DataPointFilter[] | DataGridCategories;
    column: DataPointFilter[] | DataGridCategories;
    cell: DataResultCellFilter;
    modify?: DataPointModifier[];
    sort?: DataPointSortation[];
    transform?: DataGridTransformation[];
};
export declare type ResultCellInfo = {
    value: ResultCell;
    tags: Tag[];
};
export declare type DataResultCellFilter = {
    (points: DataPoint[]): ResultCell | DataPoint[];
};
export declare type DataPointModifier = {
    applyToLevel?: number[];
    executionOrder?: number;
    cb?: any;
    callbacks?: any[];
};
export declare type DataPointSortation = {
    cb?: any;
    section: DataPointTarget;
};
export declare type DataGridTransformation = {
    cb?: any;
    section: DataPointTarget;
};
export declare type CellKey = {
    [key: string]: boolean;
};
export declare type CellKeys = {
    [key: string]: CellKey;
};
export declare type ResultColumn = {
    key: string;
    value: DataPoint[];
    style?: TableRowStyle | ChartValueStyle;
    getPoint: (index?: number) => DataPoint;
};
export declare type ResultRow = {
    key: string;
    cols: ResultColumn[];
    getColumn: (colId: number) => ResultColumn;
};
export declare type Result = {
    isValid: () => boolean;
    body: ResultRow[];
    info: ResultInfo;
    transform: TransformResult;
};
export declare type Tagger = {
    tags: DataTag[];
    push: (cat: string, val: string) => void;
};
export declare type MetaMap = {
    [key: string]: string[];
};
export declare type MetaParam = {
    cb: (row: ResultRow, params: MetaParam, result: ResultClass) => TableRowStyle[] | ChartValueStyle[];
    styles: MetaParamStyle[];
};
export declare type MetaParamStyle = {
    key?: string;
    value?: string;
    keys?: string[];
    style: TableRowStyle | ChartValueStyle;
};
export declare type Overcodes = {
    prefix?: string;
    match?: string;
    key: string;
    callback?: (label: string) => boolean;
};
export declare type RawRow = ResultCell[];
export declare type RawResultMeta = {
    key: string;
    label: string;
    data?: RawRow | RawRow[];
    info?: RawResultInfo[];
};
export declare type RawResultInfo = {
    key: string;
    value: string;
    info?: string;
};
export declare type RawResultNestedParent = {
    level: number;
    key: string;
    label: string;
};
export declare type NestedParentValue = {
    label: string;
    value: ResultCell;
};
export declare type RawResultNestedItem = {
    label: string;
    parents: RawResultNestedParent[];
};
export declare type RawResultData = {
    info: RawResultInfo[];
    header: RawRow[];
    body: RawRow[];
    meta: RawResultMeta[];
    nested?: RawResultNestedItem[];
};
export declare type RawColumnSlice = {
    label: string;
    start: number;
    end: number;
    length: number;
};
export declare type RawTable = {
    label: string;
    rows: string[];
    columns: string[];
    data: ResultCell[][];
    meta: RawResultMeta[];
};
export declare type DataPointTarget = 'row' | 'column';
export declare type RenameLabel = {
    target?: DataPointTarget;
    targets?: DataPointTarget[];
    replace: string;
    by: string;
    cb?: (label: string) => string;
    isPattern: boolean;
    ucFirst?: boolean;
};
export declare type ModifierCommandArgument = ModArgsCustom | ModArgsFilter | ModArgsFilterNested | ModArgsExclude | ModArgsStringTolabel | ModArgsTagTolabel | ModArgsAddToOthers | ModArgsAddMeta | ModArgsAddPointInfo | ModArgsMap | ModArgsRename;
export declare type ModArgsCustom = {
    key: string;
    args: any;
};
export declare type ModArgsFilter = {
    key: string;
    values: string[];
    replace?: boolean;
    origin?: boolean;
};
export declare type ModArgsFilterNested = {
    values: string[];
    hideParents: boolean;
    hideChildren: boolean;
};
export declare type ModArgsExclude = {
    key: string;
    values: string[];
    excludeAll: boolean;
};
export declare type ModArgsStringTolabel = {
    value: string;
    target: DataPointTarget;
};
export declare type ModArgsTagTolabel = {
    categoryId: number;
    target: DataPointTarget;
    glue?: string;
};
export declare type ModArgsAddToOthers = {
    push?: number;
    match: DataPointTarget;
    mode: string;
};
export declare type ModArgsAddToNew = {
    key: DataPointTarget;
    values: string[];
    alias: string;
    mode: string;
};
export declare type ModArgsCalcDifference = {
    match: DataPointTarget;
    item: string;
    mode: string;
    revert: boolean;
};
export declare type ModArgsCalcSum = {
    match: DataPointTarget;
    items?: string[];
    mode: string;
    alias: string;
};
export declare type AggregatePoints = {
    alias: string;
    key: string;
    points: DataPoint[];
};
export declare type ModArgsAddMeta = {
    key: string;
    glue?: string;
    replace?: boolean;
    append?: boolean;
};
export declare type ModArgsAddPointInfo = {
    key: string;
};
export declare type ModArgsMap = {
    source: number | DataPointTarget;
    target: DataPointTarget;
};
export declare type ModArgsRename = {
    renameStack: RenameLabel[];
};
export declare type ModArgsTranspose = {};
